package eu.mhsl.minecraft.WorldMuseum;

import eu.mhsl.minecraft.WorldMuseum.listener.ChatListener;
import eu.mhsl.minecraft.WorldMuseum.listener.PlayerListener;
import eu.mhsl.minecraft.WorldMuseum.viewableWorld.ViewableWorld;
import eu.mhsl.minecraft.WorldMuseum.viewableWorld.ViewableWorldListener;
import org.bukkit.Bukkit;
import org.bukkit.event.Listener;
import org.bukkit.generator.ChunkGenerator;
import org.bukkit.plugin.java.JavaPlugin;
import org.jetbrains.annotations.NotNull;

import java.io.File;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.Objects;
import java.util.logging.Logger;

public class Main extends JavaPlugin {
    private static Main instance;

    private final List<ViewableWorld> worlds = new ArrayList<>();

    @Override
    public void onEnable() {
        Main.instance = this;
        this.saveDefaultConfig();

        getServer().getMessenger().registerOutgoingPluginChannel(this, "BungeeCord");

        List<Listener> listeners = List.of(
            new PlayerListener(),
            new ChatListener(),
            new ViewableWorldListener()
        );
        listeners.forEach(listener -> Bukkit.getPluginManager().registerEvents(listener, this));

        this.loadWorlds();
    }

    @Override
    public void onDisable() {
        this.worlds.forEach(ViewableWorld::unloadWorld);
    }

    @Override
    public ChunkGenerator getDefaultWorldGenerator(@NotNull String worldName, String id) {
        return new ChunkGenerator() {};
    }

    private void loadWorlds() {
        File worldFolder = new File(getDataFolder().getAbsolutePath() + "/worlds");
        //if(!worldFolder.setReadOnly()) throw new RuntimeException("Could not set Worlds to readonly. Be careful!");

        //noinspection ResultOfMethodCallIgnored
        worldFolder.mkdirs();
        Arrays.stream(Objects.requireNonNull(worldFolder.listFiles())).forEach(file -> {
            if (file.isDirectory()) {
                getLogger().info("Loading world " + file.getName());
                this.worlds.add(new ViewableWorld(file));
            } else {
                getLogger().info( file.getName() + " is not a directory and therefore not a valid world!");
            }
        });
        this.worlds.forEach(ViewableWorld::loadWorld);
    }

    public static Main instance() {
        return Main.instance;
    }

    public List<ViewableWorld> getWorlds() {
        return worlds;
    }

    public static Logger logger() {
        return Main.instance().getLogger();
    }
}