package eu.mhsl.minecraft.WorldMuseum.items.actionItems;

import eu.mhsl.minecraft.WorldMuseum.Main;
import eu.mhsl.minecraft.WorldMuseum.viewableWorld.ViewableWorld;
import net.kyori.adventure.text.Component;
import net.kyori.adventure.text.format.NamedTextColor;
import net.kyori.adventure.text.serializer.plain.PlainTextComponentSerializer;
import org.bukkit.Bukkit;
import org.bukkit.Material;
import org.bukkit.entity.Player;
import org.bukkit.event.EventHandler;
import org.bukkit.event.inventory.InventoryClickEvent;
import org.bukkit.inventory.Inventory;
import org.bukkit.inventory.ItemStack;
import org.bukkit.inventory.meta.ItemMeta;
import org.jetbrains.annotations.NotNull;

import java.util.List;
import java.util.Objects;

public class WorldSelector implements IActionItem {
    private Inventory generateWorldSelectorInventory() {
        List<ViewableWorld> worlds = Main.instance().getWorlds();

        int itemCount = worlds.size() + 1;
        int inventoryRows = Math.max(itemCount / 9, 1);
        int inventorySize = inventoryRows * 9;
        Inventory inventory = Bukkit.createInventory(null, inventorySize, Component.text("Weltenmuseum"));

        worlds.stream()
            .filter(ViewableWorld::isEnabled)
            .sorted((o1, o2) -> {
                String name1 = PlainTextComponentSerializer.plainText().serialize(o1.getItem().displayName());
                String name2 = PlainTextComponentSerializer.plainText().serialize(o2.getItem().displayName());
                return name1.compareTo(name2);
            })
            .forEach(viewableWorld -> inventory.addItem(viewableWorld.getItem()));

        boolean canExit = Main.instance().getConfig().getBoolean("bungee.enabled");
        if(canExit) inventory.setItem(inventorySize - 1, exitItem());

        return inventory;
    }

    public static ItemStack exitItem() {
        ItemStack stack = ItemStack.of(Material.OAK_DOOR);
        ItemMeta meta = stack.getItemMeta();
        meta.displayName(Component.text("Zurück zum Server"));
        meta.setEnchantmentGlintOverride(true);
        stack.setItemMeta(meta);
        return stack;
    }

    @Override
    public void action(@NotNull Player player) {
        player.openInventory(generateWorldSelectorInventory());
    }

    @EventHandler
    public void onInteract(InventoryClickEvent event) {
        if(Objects.equals(event.getCurrentItem(), exitItem())) {
            LeaveItem item = new LeaveItem();
            item.action((Player) event.getWhoClicked());
            return;
        }

        List<ItemStack> worldItems = Main.instance().getWorlds().stream().map(ViewableWorld::getItem).toList();
        if(!worldItems.contains(event.getCurrentItem())) return;
        event.setCancelled(true);

        Main.instance().getWorlds().stream()
                .filter(viewableWorld -> viewableWorld.getItem().equals(event.getCurrentItem()))
                .findFirst()
                .ifPresent(viewableWorld -> viewableWorld.addViewer((Player) event.getWhoClicked()));
    }

    @Override
    public ItemStack getItem() {
        ItemStack item = new ItemStack(Material.COMPASS);
        ItemMeta meta = item.getItemMeta();
        meta.displayName(Component.text("Weltenliste", NamedTextColor.GOLD));
        item.setItemMeta(meta);
        return item;
    }

    @Override
    public int getItemLocation() {
        return 0;
    }

}
