package eu.mhsl.minecraft.WorldMuseum.listener;

import eu.mhsl.minecraft.WorldMuseum.Main;
import eu.mhsl.minecraft.WorldMuseum.items.ActionItemManager;
import eu.mhsl.minecraft.WorldMuseum.viewableWorld.ViewableWorld;
import net.kyori.adventure.util.Ticks;
import org.bukkit.Bukkit;
import org.bukkit.GameMode;
import org.bukkit.Material;
import org.bukkit.block.data.BlockData;
import org.bukkit.entity.Player;
import org.bukkit.event.EventHandler;
import org.bukkit.event.Listener;
import org.bukkit.event.block.Action;
import org.bukkit.event.block.BlockBreakEvent;
import org.bukkit.event.block.BlockPlaceEvent;
import org.bukkit.event.inventory.InventoryClickEvent;
import org.bukkit.event.player.PlayerDropItemEvent;
import org.bukkit.event.player.PlayerInteractEvent;
import org.bukkit.event.player.PlayerJoinEvent;
import org.bukkit.inventory.ItemStack;
import org.bukkit.potion.PotionEffect;
import org.bukkit.potion.PotionEffectType;

import java.util.List;

public class PlayerListener implements Listener {
    @EventHandler
    public void onJoin(PlayerJoinEvent event) {
        Player player = event.getPlayer();
        player.getInventory().clear();
        player.getActivePotionEffects().clear();
        player.clearTitle();

        player.setGameMode(GameMode.SURVIVAL);
        player.setInvulnerable(true);
        player.setFoodLevel(20);
        player.setHealth(20);
        player.setAllowFlight(true);
        player.setFlying(true);

        Main.instance().getWorlds().stream()
                .filter(ViewableWorld::isDefaultWorld)
                .findFirst()
                .orElseGet(() -> Main.instance().getWorlds().stream().findFirst().orElseThrow())
                .addViewer(player);

        event.getPlayer().addPotionEffect(
            new PotionEffect(PotionEffectType.HASTE, PotionEffect.INFINITE_DURATION, 15, false, false, false)
        );

        ActionItemManager.getInstance().givePlayerItems(event.getPlayer());
    }

    @EventHandler
    public void onBreak(BlockBreakEvent event) {
        event.setDropItems(false);
        Material material = event.getBlock().getType();
        BlockData blockData = event.getBlock().getBlockData();
        Bukkit.getScheduler().scheduleSyncDelayedTask(
            Main.instance(),
            () -> {
                event.getBlock().setType(material);
                event.getBlock().setBlockData(blockData);
            },
            Ticks.TICKS_PER_SECOND * 5
        );
    }

    @EventHandler
    public void onPlace(BlockPlaceEvent event) {
        event.setCancelled(true);
    }

    @EventHandler
    public void onDrop(PlayerDropItemEvent event) {
        event.setCancelled(true);
    }

    @EventHandler
    public void onInteract(PlayerInteractEvent event) {
        ActionItemManager actionItemManager = ActionItemManager.getInstance();

        if(event.getItem() == null) return;
        if(!List.of(Action.RIGHT_CLICK_BLOCK, Action.RIGHT_CLICK_AIR).contains(event.getAction())) return;

        Player p = event.getPlayer();
        ItemStack item = event.getItem();

        actionItemManager.getActionItems().forEach(actionItem -> {
            if (!item.equals(actionItem.getItem())) return;
            event.setCancelled(true);
            actionItem.action(p);
        });
    }

    @EventHandler
    public void onItemMove(InventoryClickEvent event) {
        event.setCancelled(true);
    }
}
